@@ -5,6 +5,7 @@ gi.require_version("GtkSource", "5")
 
 from gi.repository import Adw, Gdk, Gio, GLib, GObject, Gtk, GtkSource, Pango
 
+from dataclasses import dataclass
 import logging
 from typing import Any, Optional
 import webbrowser
@@ -27,6 +28,14 @@ from iotas.theme_selector import ThemeSelector
 from iotas.ui_utils import add_mouse_button_accel, is_likely_mobile_device
 
 
+@dataclass
+class EditorSessionDetails:
+    note: Note
+    scroll_position: float
+    cursor_position: int
+    direct_switching: Optional[bool] = None
+
+
 @Gtk.Template(resource_path="/org/gnome/World/Iotas/ui/editor.ui")
 class Editor(Adw.Bin):
     __gtype_name__ = "Editor"
@@ -86,6 +95,8 @@ class Editor(Adw.Bin):
         self.__html_generator = None
         self.__focus_mode_action = None
         self.__cancel_action = None
+        self.__previous_sessions = []
+        self.__pointer_cursor_y = None
 
         self.__hiding_bars_timeout = None
 
@@ -209,10 +220,11 @@ class Editor(Adw.Bin):
         window.connect("notify::fullscreened", self.__on_window_fullscreened)
         self._sourceview.setup()
 
-    def init_note(self, note: Note) -> None:
+    def init_note(self, note: Note, restore_session: Optional[EditorSessionDetails]) -> None:
         """Initialise editing session.
 
         :param Note note: Note to edit
+        :param EditorSessionDetails restore_session: Session to restore, optional
         """
         self.current_note = note
         buffer = self._sourceview.get_buffer()
@@ -258,25 +270,38 @@ class Editor(Adw.Bin):
             # based on configuration, the bottom bar is shown
             self._formatting_header_bar.set_visible(True)
 
-        buffer.place_cursor(buffer.get_start_iter())
+        # By popping this note off our previous sessions it allows returning from the index to
+        # switching between two notes
+        self.__flush_note_from_previous_sessions(note)
 
-        # Set scroll to top
-        self._editor_scrolledwin.get_vadjustment().set_value(0)
+        if restore_session:
+            buffer = self._sourceview.get_buffer()
+            cursor_iter = buffer.get_iter_at_offset(restore_session.cursor_position)
+            scroll_position = restore_session.scroll_position
+            GLib.idle_add(self._editor_scrolledwin.get_vadjustment().set_value, scroll_position)
+        else:
+            # Set scroll to top
+            cursor_iter = buffer.get_start_iter()
+            scroll_position = 0
+        self._editor_scrolledwin.get_vadjustment().set_value(scroll_position)
+
+        buffer.place_cursor(cursor_iter)
 
         self.__update_scheme_and_dark_style()
 
-        self._top_bar_revealer.set_reveal_child(True)
-        if (
-            iotas.config_manager.get_markdown_detect_syntax()
-            and self.__formatting_bar_visibility_setting != HeaderBarVisibility.DISABLED
-        ):
-            self._bottom_bar_revealer.set_reveal_child(True)
-        elif self._bottom_bar_revealer.get_reveal_child():
-            self._bottom_bar_revealer.set_reveal_child(False)
-        self.__update_margin_box_heights()
-        GLib.idle_add(self.__update_toolbar_underlay_margins)
+        if not (restore_session and restore_session.direct_switching):
+            self._top_bar_revealer.set_reveal_child(True)
+            if (
+                iotas.config_manager.get_markdown_detect_syntax()
+                and self.__formatting_bar_visibility_setting != HeaderBarVisibility.DISABLED
+            ):
+                self._bottom_bar_revealer.set_reveal_child(True)
+            elif self._bottom_bar_revealer.get_reveal_child():
+                self._bottom_bar_revealer.set_reveal_child(False)
+            self.__update_margin_box_heights()
+            GLib.idle_add(self.__update_toolbar_underlay_margins)
 
-        self.__check_and_hide_bars_after_delay()
+            self.__check_and_hide_bars_after_delay()
 
         if self.__focus_mode_action.get_state():
             self.__focus_mode_helper.active = True
@@ -298,9 +323,14 @@ class Editor(Adw.Bin):
             ):
                 logging.warning("Auto hiding currently provides a poor experience on mobile")
 
+        if restore_session:
+            self.__restore_session_location(restore_session)
+
     def close_note(self) -> None:
         """Close the current editing note."""
 
+        self.__store_session_details()
+
         if self.__hiding_bars_timeout:
             GLib.source_remove(self.__hiding_bars_timeout)
             self.__hiding_bars_timeout = None
@@ -376,6 +406,17 @@ class Editor(Adw.Bin):
         if self.active:
             self.__cancel_action.set_enabled(not visible)
 
+    def get_previous_session(self) -> Optional[EditorSessionDetails]:
+        """Get previous session details.
+
+        :return: Session details
+        :rtype: EditorSessionDetails, optional
+        """
+        if self.__previous_sessions:
+            return self.__previous_sessions[-1]
+        else:
+            return None
+
     @GObject.Property(type=bool, default=False)
     def active(self) -> bool:
         return self.__active
@@ -451,6 +492,14 @@ class Editor(Adw.Bin):
     ) -> None:
         self.__check_and_show_formatting_bar()
 
+    @Gtk.Template.Callback()
+    def _on_motion(self, _controller: Gtk.EventControllerMotion, _x: float, y: float) -> None:
+        self.__pointer_cursor_y = y
+
+    @Gtk.Template.Callback()
+    def _on_motion_leave(self, _controller: Gtk.EventControllerMotion) -> None:
+        self.__pointer_cursor_y = None
+
     @Gtk.Template.Callback()
     def _on_bottom_margin_clicked(
         self, gesture: Gtk.GestureClick, n_press: int, _x: float, _y: float
@@ -478,8 +527,9 @@ class Editor(Adw.Bin):
             Gdk.KEY_Shift_L,
             Gdk.KEY_Shift_R,
         ):
-            self.__check_and_hide_header_bar()
-            self.__check_and_hide_formatting_bar()
+            check_mouse_cursor = True
+            self.__check_and_hide_header_bar(check_mouse_cursor)
+            self.__check_and_hide_formatting_bar(check_mouse_cursor)
 
         return Gdk.EVENT_PROPAGATE
 
@@ -994,7 +1044,7 @@ class Editor(Adw.Bin):
             self.__updating_from_remote = False
             self.set_sensitive(True)
 
-    def __get_editor_scroll_position(self) -> None:
+    def __get_scroll_position(self) -> None:
         adj = self._editor_scrolledwin.get_vadjustment()
         value = adj.get_value()
         upper = adj.get_upper()
@@ -1004,7 +1054,7 @@ class Editor(Adw.Bin):
         else:
             return 0
 
-    def __set_editor_scroll_position(self, position: float) -> None:
+    def __set_scroll_position(self, position: float) -> None:
         adj = self._editor_scrolledwin.get_vadjustment()
         upper = adj.get_upper()
         page_size = adj.get_page_size()
@@ -1087,7 +1137,7 @@ class Editor(Adw.Bin):
                 self.__exit_render_search()
 
             self._render_edit_button_stack.set_visible_child(self._render_button)
-            self.__set_editor_scroll_position(self.__render_view.scroll_position)
+            self.__set_scroll_position(self.__render_view.scroll_position)
             self._sourceview.set_visible(True)
             self._render_edit_stack.set_visible_child(self._editor_scrolledwin)
             self.__check_and_terminate_webkit_process()
@@ -1267,7 +1317,7 @@ class Editor(Adw.Bin):
         self.focus_textview_if_editing()
         self.__update_margin_box_heights()
 
-    def __check_and_hide_formatting_bar(self) -> bool:
+    def __check_and_hide_formatting_bar(self, check_mouse_cursor_over_bar: bool = False) -> bool:
         if not self._bottom_bar_revealer.get_reveal_child():
             return False
 
@@ -1281,6 +1331,11 @@ class Editor(Adw.Bin):
         elif setting == HeaderBarVisibility.AUTO_HIDE_FULLSCREEN_ONLY:
             hide = self.get_root().is_fullscreen()
 
+        if hide and check_mouse_cursor_over_bar:
+            y = self.__pointer_cursor_y
+            if y is not None:
+                hide = y < self.get_height() - self._sourceview.get_property("top-margin")
+
         if hide:
             self.__hide_formatting_bar()
         return hide
@@ -1295,10 +1350,11 @@ class Editor(Adw.Bin):
             return
         if not iotas.config_manager.get_markdown_detect_syntax():
             return
+
         if self.__formatting_bar_visibility_setting != HeaderBarVisibility.DISABLED:
             self.__show_formatting_bar()
 
-    def __check_and_hide_header_bar(self) -> None:
+    def __check_and_hide_header_bar(self, check_mouse_cursor_over_bar: bool = False) -> None:
         if not self._top_bar_revealer.get_reveal_child():
             return
 
@@ -1308,6 +1364,11 @@ class Editor(Adw.Bin):
         elif self.__header_bar_visibility_setting == HeaderBarVisibility.AUTO_HIDE_FULLSCREEN_ONLY:
             hide = self.get_root().is_fullscreen()
 
+        if hide and check_mouse_cursor_over_bar:
+            y = self.__pointer_cursor_y
+            if y is not None:
+                hide = y > self._sourceview.get_property("top-margin")
+
         if hide:
             self.__hide_header_bar()
 
@@ -1398,7 +1459,7 @@ class Editor(Adw.Bin):
         self.__render_view.render(
             self.__note,
             None,
-            self.__get_editor_scroll_position(),
+            self.__get_scroll_position(),
         )
         self._search_header_bar.disable_actions()
         self.__focus_mode_action.set_enabled(False)
@@ -1492,3 +1553,32 @@ class Editor(Adw.Bin):
             logging.info(f"Opening URI from ctrl-click: {link}")
             webbrowser.open(link)
             self.__show_toast(_("Opening link in browser"))
+
+    def __store_session_details(self) -> None:
+        scroll_position = self._editor_scrolledwin.get_vadjustment().get_value()
+
+        buffer = self._sourceview.get_buffer()
+        insert_iter = buffer.get_iter_at_mark(buffer.get_insert())
+        cursor_location = insert_iter.get_offset()
+
+        if self.current_note in [session.note for session in self.__previous_sessions]:
+            return
+
+        details = EditorSessionDetails(
+            self.current_note,
+            scroll_position,
+            cursor_location,
+        )
+        self.__previous_sessions.append(details)
+        if len(self.__previous_sessions) > 2:
+            self.__previous_sessions = self.__previous_sessions[-2:]
+
+    def __flush_note_from_previous_sessions(self, note: Note) -> None:
+        if self.__previous_sessions and self.__previous_sessions[-1].note == note:
+            del self.__previous_sessions[-1]
+
+    def __restore_session_location(self, session: EditorSessionDetails) -> None:
+        self.__set_scroll_position(session.scroll_position)
+        buffer = self._sourceview.get_buffer()
+        cursor_iter = buffer.get_iter_at_offset(session.cursor_position)
+        buffer.place_cursor(cursor_iter)
