@@ -3,7 +3,7 @@ import gi
 gi.require_version("GtkSource", "5")
 from gi.repository import Gdk, Gio, GLib, GObject, Gtk, GtkSource
 
-from gtkspellcheck import SpellChecker
+from gtkspellcheck import SpellChecker, NoDictionariesFound
 
 import locale
 import logging
@@ -244,7 +244,14 @@ class EditorTextView(GtkSource.View):
             language = locale.getdefaultlocale()[0]
             logging.debug(f'Attempting to use locale default spelling language "{language}"')
 
-        self.__spellchecker = SpellChecker(self, language, collapse=False)
+        try:
+            self.__spellchecker = SpellChecker(self, language, collapse=False)
+        except NoDictionariesFound:
+            # Not communicating via UI for now as systems without any dictionaries are likely to
+            # be corner cases, bespoke systems eg. Arch etc
+            logging.warning("Disabling spell checker as no dictionaries were found")
+            return
+
         self.spellchecker_enabled = False
         self.__spellchecker.batched_rechecking = True
         if pref_language is not None:
