@@ -3,7 +3,7 @@ from gettext import ngettext
 from gi.repository import Adw, Gdk, GObject, Gio, GLib, Gtk
 
 import logging
-from typing import List, Optional
+from typing import Optional
 
 from iotas.category import Category, CategorySpecialPurpose
 from iotas.category_manager import CategoryManager
@@ -188,10 +188,10 @@ class Index(Adw.BreakpointBin):
             "offline", self._offline_banner, "revealed", GObject.BindingFlags.SYNC_CREATE
         )
 
-    def update_for_note_deletions(self, notes: List[Note]) -> None:
+    def update_for_note_deletions(self, notes: list[Note]) -> None:
         """Perform updates for deleted notes.
 
-        :param List[Note] notes: The deleted notes
+        :param list[Note] notes: The deleted notes
         """
         if len(notes) > 1:
             # Translators: Description, notification, {} is a positive number
@@ -842,10 +842,10 @@ class Index(Adw.BreakpointBin):
 
         return False
 
-    def __update_favourites(self, notes: List[Note], value: bool) -> None:
+    def __update_favourites(self, notes: list[Note], value: bool) -> None:
         """Change whether notes are in favourites.
 
-        :param List[Note] notes: Notes to modify
+        :param list[Note] notes: Notes to modify
         :param bool value: The new value
         """
         if self.__note_manager.set_and_persist_favourite_for_notes(notes, value):
@@ -938,10 +938,10 @@ class Index(Adw.BreakpointBin):
         self.__initial_sync_delay_complete = True
         self.__sync_manager.sync_now()
 
-    def __delete_notes(self, notes: List[Note]) -> None:
+    def __delete_notes(self, notes: list[Note]) -> None:
         """Delete the specified notes, flushing any undo-pending notes first.
 
-        :param List[Note] notes: Notes to delete
+        :param list[Note] notes: Notes to delete
         """
         self.__sync_manager.flush_pending_deletions()
         self.__note_manager.delete_notes(notes)
