@@ -74,7 +74,10 @@ class NextcloudSyncWorker(GObject.Object):
         else:
             self.verify = True
 
-        self.__headers = {"User-Agent": self.USER_AGENT}
+        self.__headers = {
+            "User-Agent": self.USER_AGENT,
+            "Cache-Control": "no-cache",
+        }
 
     def init_auth(self) -> None:
         """Initialise Nextcloud authentication details from the Secret Service."""
