@@ -5,7 +5,7 @@ from hashlib import sha256
 import logging
 from threading import Lock, Thread
 import time
-from typing import List, Optional, Tuple
+from typing import Optional, Tuple
 
 from iotas.category_manager import CategoryManager
 import iotas.config_manager
@@ -318,8 +318,8 @@ class SyncManager(GObject.Object):
         self.__finish_sync(changes_in + changes_out)
 
     def __determine_remote_new_and_updated(
-        self, notes: List[dict], remote_ids_from_db: List[int], force_refresh_all: bool
-    ) -> Tuple[List[dict], List[dict]]:
+        self, notes: list[dict], remote_ids_from_db: list[int], force_refresh_all: bool
+    ) -> Tuple[list[dict], list[dict]]:
         create = []
         update = []
         for sync_note in notes:
@@ -337,8 +337,8 @@ class SyncManager(GObject.Object):
         return (create, update)
 
     def __preprocess_remote_updates_for_locally_modified(
-        self, updates_in: List[dict]
-    ) -> List[dict]:
+        self, updates_in: list[dict]
+    ) -> list[dict]:
         updates_out = []
         for sync_note in updates_in:
             # Fetch note
@@ -382,7 +382,7 @@ class SyncManager(GObject.Object):
 
         return updates_out
 
-    def __check_for_remote_delete_conflicts(self, remote_del: List[dict]) -> None:
+    def __check_for_remote_delete_conflicts(self, remote_del: list[dict]) -> None:
         for remote_id in remote_del:
             note = self.__note_manager.fetch_note_by_remote_id(remote_id)
             if note is None:
@@ -398,7 +398,7 @@ class SyncManager(GObject.Object):
                 note.dirty = False
                 self.__db.persist_note_dirty(note)
 
-    def __apply_local_updates(self) -> List[Note]:
+    def __apply_local_updates(self) -> list[Note]:
         local_upd = [
             v
             for v in self.__note_manager.base_model
@@ -412,7 +412,7 @@ class SyncManager(GObject.Object):
             idle_add_wait(self.__update_note_metadata_post_sync, note, res)
         return local_upd
 
-    def __apply_local_creations(self) -> List[Note]:
+    def __apply_local_creations(self) -> list[Note]:
         local_new = [
             n
             for n in self.__note_manager.base_model
@@ -426,7 +426,7 @@ class SyncManager(GObject.Object):
             idle_add_wait(self.__update_note_metadata_post_sync, note, res)
         return local_new
 
-    def __apply_local_deletions(self) -> List[Note]:
+    def __apply_local_deletions(self) -> list[Note]:
         local_del_queue = [
             n
             for n in self.__note_manager.base_model
@@ -457,7 +457,7 @@ class SyncManager(GObject.Object):
             self.__lock.release()
 
     def __integrate_remote_changes(
-        self, remote_new: List[dict], remote_upd: List[dict], remote_del: List[int]
+        self, remote_new: list[dict], remote_upd: list[dict], remote_del: list[int]
     ) -> None:
         for sync_note in remote_new + remote_upd:
             existing_note = self.__note_manager.fetch_note_by_remote_id(sync_note["id"])
