@@ -1,10 +1,12 @@
 from gi.repository import Adw, Gio, GLib, GObject, Gtk
 
 import logging
+from typing import Optional
 
 import iotas.config_manager
 from iotas.category_manager import CategoryManager
 from iotas import const
+from iotas.editor import EditorSessionDetails
 from iotas.note import Note
 from iotas.note_database import NoteDatabase
 from iotas.note_manager import NoteManager
@@ -159,6 +161,11 @@ class Window(Adw.ApplicationWindow):
         action.connect("activate", self.__on_open_note)
         action_group.add_action(action)
 
+        action = Gio.SimpleAction.new("open-previous-note")
+        action.connect("activate", lambda _o, _v: self.__open_previous_note())
+        app.set_accels_for_action("win.open-previous-note", ["<Control>l"])
+        action_group.add_action(action)
+
         action = Gio.SimpleAction.new("search-from-cli", GLib.VariantType("s"))
         action.connect("activate", self.__on_search_from_cli)
         action_group.add_action(action)
@@ -217,7 +224,7 @@ class Window(Adw.ApplicationWindow):
         if self.__initialised:
             if self._editor.active:
                 self._navigation.pop()
-            self.__open_note_by_id(note_id, False)
+            self.__open_note_by_id(note_id, immediate=False)
         else:
             self.__open_note_after_init = note_id
 
@@ -261,9 +268,11 @@ class Window(Adw.ApplicationWindow):
             note = self.__note_manager.create_note(self.__category_manager.all_category)
             self.__open_note(note, immediate=True)
         elif self.__open_note_after_init is not None:
-            self.__open_note_by_id(self.__open_note_after_init, True)
+            self.__open_note_by_id(self.__open_note_after_init, immediate=True)
 
-    def __open_note(self, note: Note, immediate: bool) -> None:
+    def __open_note(
+        self, note: Note, immediate: bool, restore_session: Optional[EditorSessionDetails] = None
+    ) -> None:
         if immediate:
             self._navigation.set_animate_transitions(False)
         if not note.content_loaded:
@@ -271,7 +280,7 @@ class Window(Adw.ApplicationWindow):
         # Handle opening a note from CLI / shell search provider when another note is open
         if self._editor.active:
             self._navigation.pop()
-        self._editor.init_note(note)
+        self._editor.init_note(note, restore_session)
         self._navigation.push(self._editor_page)
         if immediate:
             self._navigation.set_animate_transitions(True)
@@ -283,6 +292,23 @@ class Window(Adw.ApplicationWindow):
             return
         self.__open_note(note, immediate)
 
+    def __open_previous_note(self) -> None:
+        session = self._editor.get_previous_session()
+        if session is None:
+            return
+        if not self.__note_manager.fetch_note_by_id(session.note.id):
+            return
+        if session.note.locally_deleted:
+            return
+        if session.note in self.__note_manager.get_deleted_notes_pending_undo():
+            return
+        if self._editor.current_note == session.note:
+            return
+
+        logging.info("Opening previous note")
+        session.direct_switching = self._editor.active
+        self.__open_note(session.note, immediate=session.direct_switching, restore_session=session)
+
     def __show_login_window(self) -> None:
         if self.__secret_service_failed:
             mode = LoginDialogMode.SECRET_SERVICE_FAILURE
